<?php

namespace PlugnaLAI;

class PlugnaLAI_AJAX
{

    protected $plugin_dir;
    /**
     * @var PlugnaLAI_Helpers
     */
    protected $helpers;

    /**
     * @var PlugnaLAI_DB
     */
    protected $db;

    /**
     * @var PlugnaLAI_API
     */
    protected $api;

    public function __construct($plugin_dir, $helpers, $db, $api)
    {
        $this->plugin_dir = $plugin_dir;
        $this->helpers = $helpers;
        $this->db = $db;
        $this->api = $api;

        //AJAX
        add_action( 'wp_ajax_plugnalai_prompt', array(&$this, 'ajax_prompt') );
        add_action( 'wp_ajax_nopriv_plugnalai_prompt', array(&$this, 'ajax_prompt') );

        add_action( 'wp_ajax_plugnalai_remove_style', array(&$this, 'ajax_remove_style') );
        add_action( 'wp_ajax_nopriv_plugnalai_remove_style', array(&$this, 'ajax_remove_style') );

        add_action( 'wp_ajax_plugnalai_like_style', array(&$this, 'ajax_like_style') );
        add_action( 'wp_ajax_nopriv_plugnalai_like_style', array(&$this, 'ajax_like_style') );

        add_action( 'wp_ajax_plugnalai_publish_styles', array(&$this, 'ajax_publish_style') );
        add_action( 'wp_ajax_nopriv_plugnalai_publish_styles', array(&$this, 'ajax_publish_style') );

        add_action( 'wp_ajax_plugnalai_edit_saved_styles', array(&$this, 'ajax_edit_saved_styles') );
        add_action( 'wp_ajax_nopriv_plugnalai_edit_saved_styles', array(&$this, 'ajax_edit_saved_styles') );

        add_action( 'wp_ajax_plugnalai_check_credits', array(&$this, 'ajax_check_credits') );
        add_action( 'wp_ajax_nopriv_plugnalai_check_credits', array(&$this, 'ajax_check_credits') );


        //AJAX
        add_action('admin_enqueue_scripts', array(&$this, 'plugna_ail_register_ajax_script'));


        add_action('wp_ajax_plugna_ail_check_credits', array(&$this, 'ajax_check_credits')); //TODO: duplicate
        add_action('wp_ajax_plugna_ail_register', array(&$this, 'plugna_ail_register_ajax'));
        add_action('wp_ajax_plugna_ail_finalize_registration', array(&$this, 'plugna_ail_finalize_ajax'));
        add_action('wp_ajax_plugna_ail_redeem', array(&$this, 'plugna_ail_redeem'));
        add_action('wp_ajax_plugna_ail_unregister_site', array(&$this, 'plugna_ail_unregister_site'));
    }

    public function ajax_check_credits(){
        $this->helpers->check_access();

        $credits = $this->api->check_credits();

        if(is_wp_error($credits)) {
            $this->error($credits->get_error_message());
        }

        $this->db->save_credits_to_db($credits);

        if(is_numeric($credits)) {
            $this->success($credits);
        }else{
            $this->error($credits);
        }

        wp_send_json( array(
            'success' => is_numeric($credits),
            'message' => $credits
        ) );
    }

    public function ajax_publish_style(){
        $this->helpers->check_access();

        $this->db->publish_styles($_POST['ids'], $_POST['old_styles_included'] == 'true');

        $this->success();
    }

    public function ajax_like_style(){
        $this->helpers->check_access();

        if(!is_numeric($_POST['id']) || !isset($_POST['like'])){
            return;
        }

        $like = $_POST['like'];

        $prompt = $this->db->get_last_prompt();
        if($prompt) {
            $response = $this->api->feedback($prompt['question'], $prompt['answer'], $like);
            if(is_wp_error($response)) {
                $this->error($response->get_error_message());
            }
        }

        $this->success_with_credits('', $response['credits']);
    }

    public function ajax_remove_style(){
        $this->helpers->check_access();

        if(!is_numeric($_POST['id'])){
            return;
        }

        $this->db->delete_temporary_prompt($_POST['id']);

        $this->success();
    }

    public function ajax_prompt() {

        $this->helpers->check_access();
        $question = $_POST['text'];
        $id = $_POST['id'];

        $response = $this->api->prompt($question);


        if(is_wp_error($response)) {
            $this->error($response->get_error_message());
        }

        if(!$response['success']){
            $this->error($response['message']);
        }

        $this->db->save_temporary_prompts($question, $response['answer'], $id, $response['credits']);

        //TODO: Add nonce validation
        //TODO: Do something with the text message, such as validate it or process it.

        $this->success_with_credits($response['answer'], $response['credits']);
    }

    public function ajax_edit_saved_styles(){

        $this->helpers->check_access();

        $this->db->edit_saved_styles();

        $this->success();
    }

    // Register the AJAX endpoint

    public function plugna_ail_register_ajax()
    {
        $this->helpers->check_access();

        check_ajax_referer('plugna_ail_register', 'nonce');

        // Get the email, site domain, and OpenAI key values from the AJAX request
        $email = sanitize_email($_POST['email']);
        $openai_key = sanitize_text_field($_POST['openai_key']);

        if(empty($email)) {
            $this->error('Please enter your email address.');
            return;
        }

        if (!is_email($email)) {
            $this->error('Please enter a valid email address.');
            return;
        }

        if(empty($openai_key)) {
            $this->error('Please enter your OpenAI key.');
            return;
        }

        $response = $this->api->register($email, $openai_key);

        if(is_wp_error($response)) {
            $this->error($response->get_error_message());
        }

        if ($response['success']) {

            update_option('plugnalai_status', 'pending');
            update_option('plugnalai_secret', $response['message']);
            update_option('plugnalai_email', $email);

            $this->success(__('Your registration was successful.', 'plugnalai'));
        } else {
            $this->error($response['message']);
        };
    }

// Register the AJAX endpoint
    public function plugna_ail_finalize_ajax()
    {
        $this->helpers->check_access();

        check_ajax_referer('plugna_ail_finalize_registration', 'nonce');

        $response = $this->api->check_site();

        if(is_wp_error($response)) {
            $this->error($response->get_error_message());
        }

        if ($response['success']) {
            update_option('plugnalai_status', 'active');
            $this->success(__('Your registration was successful.', 'plugnalai'));
        } else {
            $this->error($response['message']);
        };
    }


    public function plugna_ail_redeem()
    {
        $this->helpers->check_access();
        check_ajax_referer('plugna_ail_redeem', 'nonce');

        $lk = sanitize_text_field($_POST['license_key']);

        if (empty($lk)) {
            $this->error(__('Please enter a valid license key.', 'plugnlai'));
            return;
        }

        $response = $this->api->apply_license_key($lk);

        if(is_wp_error($response)) {
            $this->error($response->get_error_message());
        }

        PlugnaLAI_DB::save_credits_to_db($response['message']);

        // Send the API response as a JSON-encoded object back to the frontend
        wp_send_json($response);
    }


    public function plugna_ail_unregister_site()
    {
        $this->helpers->check_access();

        check_ajax_referer('plugna_ail_unregister_site', 'nonce');

        delete_option('plugnalai_status');
        delete_option('plugnalai_secret');

        $this->success(__('Site unregistered successfully.', 'plugnalai'));
    }

    // Add the AJAX script to the settings page
    public function plugna_ail_register_ajax_script()
    {
        wp_enqueue_script('plugna-ail-register-ajax', plugin_dir_url(__FILE__) . 'js/register-ajax.js', array('jquery'), '1.0', true);
        wp_localize_script('plugna-ail-register-ajax', 'plugnaAilAjax', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('plugna_ail_register'),
            'nonce_finalize' => wp_create_nonce('plugna_ail_finalize_registration'),
            'nonce_unregister_site' => wp_create_nonce('plugna_ail_unregister_site'),
            'nonce_redeem' => wp_create_nonce('plugna_ail_redeem'),
            'nonce_check_credits' => wp_create_nonce('plugna_ail_check_credits'),
            'success' => __('Your registration was successful.', 'plugna_ail'),
            'error' => __('Failed to register. Please try again.', 'plugna_ail')
        ));
    }

    protected function success($message = ''){
        wp_send_json( array('success' => true, 'message' => $message) );
    }

    protected function error($message = ''){
        wp_send_json( array('success' => false, 'message' => $message) );
    }

    protected function success_with_credits($message = '', $credits = 0){
        wp_send_json( array(
            'success' => true,
            'credits' => $credits,
            'message' => $message
        ) );
    }

}