<?php

namespace PlugnaLAI;

//require_once plugin_dir_path(__FILE__) . '../vendor/autoload.php';

class PlugnaLAI_API
{
    protected $api_key;
    protected $helpers;

    public function __construct($helpers)
    {
        $this->helpers = $helpers;
    }

    public function prompt($text_message) {
        $res = $this->sendOpenAI($text_message);

        if(is_wp_error($res)){
            return $res;
        }

        if(!$res['success']){
            return $res;
        }

        return [
            'success' => $res['success'],
            'answer' => $this->prepare_response($res['message']),
            'credits' => $res['credits']
        ];
    }

    public function check_credits(){
        return $this->post('/check-credit', array())['message'];
    }

    public function feedback($question, $answer, $like = false){
        return $this->post('/feedback', array(
            'question' => $question,
            'answer' => $answer,
            'like' => $like
        ));
    }

    protected function prepare_response($response = '') {
        return str_replace(array("\r", "\n"), '', $response);
    }

    protected function sendOpenAI($message = null){

        if(!$message) {
            return false;
        }

       return $this->post('/get-css', array(
           'message' => $message
       ));
    }

    public function check_site(){
        return $this->post('/check-site');
    }

    public function apply_license_key($lk){
        return $this->post('/apply-license-key', array(
            'license_key' => $lk
        ));
    }

    public function register($email, $openAIKey){
        return $this->post('/register', array(
            'email' => $email,
            'openai_key' => $openAIKey
        ));
    }

    protected function post($endpoint, $data = []){
        $postData = array(
            'site' => $_SERVER['SERVER_NAME'],
            'secret' => get_option('plugnalai_secret'),
        );
        $postData = array_merge($postData, $data);
        $result =  wp_remote_post(PLUGNALAI_ENDPOINT. $endpoint, [
            'headers' => array(
                'Content-Type' => 'application/json'
            ),
            'timeout' => 30, // set timeout to 30 seconds
            'body' => json_encode($postData)
        ]);

        if(is_wp_error($result)) {
            if(
                $this->helpers->does_text_contains_any_substring(
                    strtolower($result->get_error_message()),
                    array('connection refused', 'failed to connect', 'curl error: 7')
                )){
                return [
                    'code' => 502,
                    'message' => 'Unable to connect to Plugna Login AI server. Please try again later.',
                    'success' => false,
                ];
            }
            return $result;
        }

        $data = json_decode($result['body'], true);

        return [
            'code' => $result['response']['code'],
            'message' => $data['message'],
            'success' => $data['success'],
            'credits' => (int) $result['headers']->offsetGet('credits')
        ];
    }
}