<?php

namespace PlugnaLAI;

class PlugnaLAI_DB
{
    public function edit_saved_styles(){
        $css = get_option('plugnalai_css');
        if(!$css){
            return;
        }
        $css = json_decode($css, true);
        //move all saved styles to temporary prompts
        $prompts = [];
        $counter = 1;

        foreach ((array) $css['css'] as $p){
            $prompts[$counter] = array(
                'question' => $p['question'],
                'answer' => $p['answer'],
            );
            $counter++;
        }

        foreach ((array) $css['prompts'] as $p){
            $prompts[$counter] = array(
                'question' => $p['question'],
                'answer' => $p['answer'],
            );
            $counter++;
        }

        $css['prompts'] = $prompts;
        unset($css['css']);
        update_option('plugnalai_css', json_encode($css));
    }

    public static function save_credits_to_db($credits){

        if(!is_numeric($credits)){
            return;
        }

        $css = get_option('plugnalai_css');

        if (empty($css)) {
            $css = array(
                'css' => array(), //for saved css
                'prompts' => array() //for temporary prompts
            );
        }else {
            $css = json_decode($css, true);
        }

        $css['credits'] = $credits;

        update_option('plugnalai_css', json_encode($css));
    }

    public function publish_styles($ids = array(), $include_old_styles = true){
        $css = get_option('plugnalai_css');
        if(!$css){
            return;
        }
        $css = json_decode($css, true);

        if(!$include_old_styles){
            $css['css'] = array();
        }

        foreach ($ids as $id){
            if(!empty($css['prompts'][$id])){
                $css['css'][] = array(
                    'question' => $css['prompts'][$id]['question'],
                    'answer' => $css['prompts'][$id]['answer'],
                );
            }
        }
        unset($css['prompts']);
        update_option('plugnalai_css', json_encode($css));
        $version = (int) get_option('plugnalai_version');
        update_option('plugnalai_version', $version + 1);
    }

    public function get_last_prompt(){
        $css = get_option('plugnalai_css');
        if(!$css){
            return;
        }
        $css = json_decode($css, true);

        return end($css['prompts']);
    }

    public function delete_temporary_prompt($id){
        $css = get_option('plugnalai_css');
        if(!$css){
            return;
        }
        $css = json_decode($css, true);

        unset($css['prompts'][$id]);

        update_option('plugnalai_css', json_encode($css));
    }

    public function save_temporary_prompts($question = '', $answer = '', $id = '', $credits = ''){
        $css = get_option('plugnalai_css');
        if(!$css){
            $css = array(
                'css' => array(), //for saved css
                'prompts' => array() //for temporary prompts
            );
        }
        $css = json_decode($css, true);

        if($credits){
            $css['credits'] = $credits;
        }

        $css['prompts'][$id] = array(
            'question' => $question,
            'answer' => $answer,
        );

        update_option('plugnalai_css', json_encode($css));
    }

}