<?php

namespace PlugnaLAI;

require_once plugin_dir_path(__FILE__) . 'PlugnaLAI_API.php';
require_once plugin_dir_path(__FILE__) . 'PlugnaLAI_Helpers.php';
require_once plugin_dir_path(__FILE__) . 'PlugnaLAI_AJAX.php';
require_once plugin_dir_path(__FILE__) . 'PlugnaLAI_DB.php';

class PlugnaLAI_Plugin
{
    protected $api;
    protected $plugin_dir;
    protected $version;
    protected $helpers;
    protected $ajax;
    protected $db;

    public function get_version()
    {
        return $this->version;
    }

    public function __construct($plugin_dir, $version)
    {
        $this->plugin_dir = $plugin_dir;
        $this->version = $version;

        $this->helpers = new PlugnaLAI_Helpers;
        $this->db = new PlugnaLAI_DB;
        $this->api = new PlugnaLAI_API($this->helpers);
        $this->ajax = new PlugnaLAI_AJAX(
                $plugin_dir,
                $this->helpers,
                $this->db,
                $this->api
        );

        if(is_admin()){
            require_once $plugin_dir . 'config.php';
        }

        self::adminMenu();

        add_action('login_enqueue_scripts', array(&$this, 'enqueue'));
        add_action('login_head', array(&$this, 'login_head'));
        add_action('init', array(&$this, 'init'));
        add_action('template_redirect', array($this, 'generate_css') );
        add_action('admin_menu', array(&$this, 'register_plugin_settings_page'));
        add_filter('admin_body_class', array(&$this, 'body_class'));
        add_filter('plugin_action_links_plugna-login-ai/plugin.php', array(&$this, 'plugin_actions_links'));

    }

    public static function pluginDir($url)
    {
        return trailingslashit(plugin_dir_url(__DIR__)) . $url;
    }

    public function init(){
        add_rewrite_endpoint( 'plugnalai_css', EP_ROOT );
    }

    public function render_plugin_settings_page()
    {
        require_once $this->plugin_dir . 'templates/settings_page.php';
    }

    public function register_plugin_settings_page()
    {
        add_submenu_page(
            'tools.php',    // parent slug
            'Plugna - Login AI', // page title
            'Plugna - Login AI', // menu title
            'manage_options', // capability required to access the page
            'plugna-login-ai', // menu slug
            array(&$this, 'render_plugin_settings_page') // callback function to render the page
        );
    }

    public function plugin_actions_links($links){
        $settings_link = '<a href="/wp-admin/tools.php?page=plugna-login-ai">' . __('Settings') . '</a>';
        array_push($links, $settings_link);
        return $links;
    }

    public function body_class($classes)
    {
        // Add a custom class to the body tag in the WordPress admin area

        $status = get_option('plugnalai_status');
        if ($status === 'pending') {
            $classes .= ' plugnalai-pending-verification';
        } else if ($status === 'active') {
            $classes .= ' plugnalai-registered';
        }
        return $classes;
    }




    public function generate_css(){
        global $wp_query;

        if ( isset( $wp_query->query_vars['plugnalai_css'] ) ) {
            $this->print_css();
            exit;
        }
        die;
    }

    public function print_css(){

        $css = get_option('plugnalai_css');


        if (empty($css)) {
            exit;
        }
        $css = json_decode($css, true);

        if (empty($css['css'])) {
            exit;
        }

        $styles = '';

        foreach ((array)$css['css'] as $style) {
           $styles .= $style['answer'];
        }

        header('Content-Type: text/css; charset=utf-8');
        header('Cache-Control: public, max-age=31536000'); // 1 year cache

        echo wp_kses_post($styles);

    }


    public function login_head(){
        ?>
        <link rel="stylesheet" id="plugnalai-css" href="/?plugnalai_css=<?php echo get_option('plugnalai_version') ?>&amp;ver=<?php echo $this->get_version() ?>"/>
        <div id="plugnalai-styles-container"></div>
        <?php

    }


    public function enqueue() {
        if(!$this->helpers->is_plugna_context_login_page()) {
            return;
        }

        if(!$this->helpers->does_user_has_access()) {
            return;
        }

        wp_enqueue_script('jquery-ui-draggable');
        wp_enqueue_script( 'plugnalai-script', self::pluginDir('js/plai.js'), array(), $this->get_version(), true );
        wp_localize_script( 'plugnalai-script', 'SLAjax', array(
            'ajax_url' => admin_url( 'admin-ajax.php' )
        ));

        wp_enqueue_style('plai-styles', self::pluginDir('css/plai.css'), array(), $this->get_version());


        echo '<div id="plugnalai-previous-prompts" style="display: none">'.get_option('plugnalai_css').'</div>';

    }

    public static function adminMenu()
    {
        //Run only on wp-admin
        if (is_admin()) {
            add_action('admin_menu', 'Plugna::admin_menu');
        }
    }

}